import os

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QDialog, QLabel, QLineEdit, QPushButton, QFileDialog, QMessageBox


class ConfigWindow(QDialog):
	"""
	Server settings window class
	"""
	
	def __init__(self, config):
		super().__init__()
		self.config = config
		self.initUI()
	
	def initUI(self):
		"""
		Settings window
		:return:
		"""
		self.setFixedSize(365, 260)
		self.setWindowTitle('Настройки сервера')
		self.setAttribute(Qt.WA_DeleteOnClose)
		self.setModal(True)
		
		# Надпись о файле базы данных:
		self.db_path_label = QLabel('Путь до файла базы данных: ', self)
		self.db_path_label.move(10, 10)
		self.db_path_label.setFixedSize(240, 15)
		
		# Строка с путём базы
		self.db_path = QLineEdit(self)
		self.db_path.setFixedSize(250, 20)
		self.db_path.move(10, 30)
		self.db_path.setReadOnly(True)
		
		# Кнопка выбора пути.
		self.db_path_select = QPushButton('Обзор...', self)
		self.db_path_select.move(275, 28)
		
		# Метка с именем поля файла базы данных
		self.db_file_label = QLabel('Имя файла базы данных: ', self)
		self.db_file_label.move(10, 68)
		self.db_file_label.setFixedSize(180, 15)
		
		# Поле для ввода имени файла
		self.db_file = QLineEdit(self)
		self.db_file.move(200, 66)
		self.db_file.setFixedSize(150, 20)
		
		# Метка с номером порта
		self.port_label = QLabel('Номер порта для соединений:', self)
		self.port_label.move(10, 108)
		self.port_label.setFixedSize(180, 15)
		
		# Поле для ввода номера порта
		self.port = QLineEdit(self)
		self.port.move(200, 108)
		self.port.setFixedSize(150, 20)
		
		# Метка с адресом для соединений
		self.ip_label = QLabel('С какого IP принимаем соединения:', self)
		self.ip_label.move(10, 148)
		self.ip_label.setFixedSize(180, 15)
		
		# Метка с напоминанием о пустом поле.
		self.ip_label_note = QLabel(
			' оставьте это поле пустым, чтобы\n принимать соединения с любых адресов.',
			self)
		self.ip_label_note.move(10, 168)
		self.ip_label_note.setFixedSize(500, 30)
		
		# Поле для ввода ip
		self.ip = QLineEdit(self)
		self.ip.move(200, 148)
		self.ip.setFixedSize(150, 20)
		
		# Кнопка сохранения настроек
		self.save_btn = QPushButton('Сохранить', self)
		self.save_btn.move(190, 220)
		
		# Кнапка закрытия окна
		self.close_button = QPushButton('Закрыть', self)
		self.close_button.move(275, 220)
		self.close_button.clicked.connect(self.close)
		
		self.db_path_select.clicked.connect(self.open_file_dialog)
		
		self.show()
		
		self.db_path.insert(self.config['SETTINGS']['Database_path'])
		self.db_file.insert(self.config['SETTINGS']['Database_file'])
		self.port.insert(self.config['SETTINGS']['Default_port'])
		self.ip.insert(self.config['SETTINGS']['Listen_Address'])
		self.save_btn.clicked.connect(self.save_server_config)
	
	def open_file_dialog(self):
		"""
		Folder selection window handler method
		:return:
		"""
		global dialog
		dialog = QFileDialog(self)
		path = dialog.getExistingDirectory()
		path = path.replace('/', '\\')
		self.db_path.clear()
		self.db_path.insert(path)
	
	def save_server_config(self):
		"""
		The method for saving the settings.
		Checks the correctness of the entered data and
		if everything is correct, it saves the ini file.
		:return:
		"""
		global config_window
		message = QMessageBox()
		self.config['SETTINGS']['Database_path'] = self.db_path.text()
		self.config['SETTINGS']['Database_file'] = self.db_file.text()
		try:
			port = int(self.port.text())
		except ValueError:
			message.warning(self, 'Ошибка', 'Порт должен быть числом')
		else:
			self.config['SETTINGS']['Listen_Address'] = self.ip.text()
			if 1023 < port < 65536:
				self.config['SETTINGS']['Default_port'] = str(port)
				dir_path = os.path.dirname(os.path.realpath(__file__))
				dir_path = os.path.join(dir_path, '../../..')
				with open(f"{dir_path}/{'server.ini'}", 'w') as conf:
					self.config.write(conf)
					message.information(
						self, 'OK', 'Настройки успешно сохранены!')
			else:
				message.warning(
					self, 'Ошибка', 'Порт должен быть от 1024 до 65536')



#
import binascii
import hmac
import json
import os
import select
import socket
import threading

from chatserver.сhatserver.Log.Log_decorators import login_required
from chatserver.сhatserver.Log.server_log_config import server_log
from chatserver.сhatserver.Mainlib.JIM import get_message, send_message
from chatserver.сhatserver.Mainlib.descriptors import Port, IPAddress
from chatserver.сhatserver.Mainlib.variables import DESTINATION, SENDER, ACTION, PRESENCE, TIME, USER, MESSAGE_TEXT, \
	MESSAGE, RESPONSE_200, RESPONSE_400, ERROR, EXIT, ACCOUNT_NAME, GET_CONTACTS, RESPONSE_202, LIST_INFO, ADD_CONTACT, \
	REMOVE_CONTACT, ONLINE_USERS_REQUEST, USERS_REQUEST, PUBLIC_KEY_REQUEST, RESPONSE_511, DATA, RESPONSE, PUBLIC_KEY, \
	RESPONSE_205


class MessageProcessor(threading.Thread):
	"""
	The main server class. Accepts connections, dictionaries - packages
    from clients, processes incoming messages.
    Works as a separate thread.
	"""
	port = Port()
	addr = IPAddress()
	
	def __init__(self, listen_address, listen_port, database):
		# Параментры подключения
		self.addr = listen_address
		self.port = listen_port
		
		# База данных сервера
		self.database = database
		
		# Сокет, через который будет осуществляться работа
		self.sock = None
		
		# Список подключённых клиентов.
		self.clients = []
		
		# Сокеты
		self.listen_sockets = None
		self.error_sockets = None
		
		# Флаг продолжения работы
		self.running = True
		
		# Словарь содержащий сопоставленные имена и соответствующие им сокеты.
		self.names = dict()
		
		# Конструктор предка
		super().__init__()
	
	def run(self):
		"""
		Thread main loop function
		:return:
		"""
		# Инициализация Сокета
		self.init_socket()
		
		# Основной цикл программы сервера
		while self.running:
			# Ждём подключения, если таймаут вышел, ловим исключение.
			try:
				client, client_address = self.sock.accept()
			except OSError:
				pass
			else:
				server_log.info(f'Установлено соедение с ПК {client_address}')
				client.settimeout(5)
				self.clients.append(client)
			
			recv_data_lst = []
			send_data_lst = []
			err_lst = []
			# Проверяем на наличие ждущих клиентов
			try:
				if self.clients:
					recv_data_lst, self.listen_sockets, self.error_sockets = select.select(
						self.clients, self.clients, [], 0)
			except OSError as err:
				server_log.error(f'Ошибка работы с сокетами: {err.errno}')
			
			# принимаем сообщения и если ошибка, исключаем клиента.
			if recv_data_lst:
				for client_with_message in recv_data_lst:
					try:
						self.process_client_message(
							get_message(client_with_message), client_with_message)
					except (OSError, json.JSONDecodeError, TypeError) as err:
						server_log.debug(f'Getting data from client exception.', exc_info=err)
						self.remove_client(client_with_message)
	
	def remove_client(self, client):
		"""
		Client handler method with which the connection was interrupted.
        Searches for a client and removes him from the lists and base
		:param client:
		:return:
		"""
		server_log.info(f'Клиент {client.getpeername()} отключился от сервера.')
		for name in self.names:
			if self.names[name] == client:
				self.database.user_logout(name)
				del self.names[name]
				break
		self.clients.remove(client)
		client.close()
	
	def init_socket(self):
		"""Server socket initialize"""
		server_log.info(
			f'Запущен сервер, порт для подключений: {self.port}, адрес с которого принимаются подключения: {self.addr}.'
			f' Если адрес не указан, принимаются соединения с любых адресов.')
		# Готовим сокет
		transport = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		transport.bind((self.addr, self.port))
		transport.settimeout(0.5)
		
		# Начинаем слушать сокет.
		self.sock = transport
		self.sock.listen(10240)
	
	def process_message(self, message):
		"""
		Method of sending message to client
		:param message:
		:return:
		"""
		if message[DESTINATION] in self.names and self.names[message[DESTINATION]
		] in self.listen_sockets:
			try:
				send_message(self.names[message[DESTINATION]], message)
				server_log.info(
					f'Отправлено сообщение пользователю {message[DESTINATION]} от пользователя {message[SENDER]}.')
			except OSError:
				self.remove_client(message[DESTINATION])
		elif message[DESTINATION] in self.names and self.names[message[DESTINATION]] not in self.listen_sockets:
			server_log.error(
				f'Связь с клиентом {message[DESTINATION]} была потеряна. Соединение закрыто, доставка невозможна.')
			self.remove_client(self.names[message[DESTINATION]])
		else:
			server_log.error(
				f'Пользователь {message[DESTINATION]} не зарегистрирован на сервере, отправка сообщения невозможна.')
	
	@login_required
	def process_client_message(self, message, client):
		"""
		Function for processing incoming messages.
		:param message:
		:param client:
		:return:
		"""
		server_log.debug(f'Разбор сообщения от клиента : {message}')
		# Если это сообщение о присутствии, принимаем и отвечаем
		if ACTION in message and message[ACTION] == PRESENCE and TIME in message and USER in message:
			# Если сообщение о присутствии то вызываем функцию авторизации.
			self.autorize_user(message, client)
		
		# Если это сообщение, то отправляем его получателю.
		elif ACTION in message and message[ACTION] == MESSAGE and DESTINATION in message and TIME in message \
				and SENDER in message and MESSAGE_TEXT in message and self.names[message[SENDER]] == client:
			if message[DESTINATION] in self.names:
				self.database.process_message(
					message[SENDER], message[DESTINATION])
				self.process_message(message)
				try:
					send_message(client, RESPONSE_200)
				except OSError:
					self.remove_client(client)
			else:
				response = RESPONSE_400
				response[ERROR] = 'Пользователь не зарегистрирован на сервере.'
				try:
					send_message(client, response)
				except OSError:
					pass
			return
		
		# Если клиент выходит
		elif ACTION in message and message[ACTION] == EXIT and ACCOUNT_NAME in message \
				and self.names[message[ACCOUNT_NAME]] == client:
			self.remove_client(client)
		
		# Если это запрос контакт-листа
		elif ACTION in message and message[ACTION] == GET_CONTACTS and USER in message and \
				self.names[message[USER]] == client:
			response = RESPONSE_202
			response[LIST_INFO] = self.database.get_contacts(message[USER])
			try:
				send_message(client, response)
			except OSError:
				self.remove_client(client)
		
		# Если это добавление контакта
		elif ACTION in message and message[ACTION] == ADD_CONTACT and ACCOUNT_NAME in message and USER in message \
				and self.names[message[USER]] == client:
			self.database.add_contact(message[USER], message[ACCOUNT_NAME])
			try:
				send_message(client, RESPONSE_200)
			except OSError:
				self.remove_client(client)
		
		# Если это удаление контакта
		elif ACTION in message and message[ACTION] == REMOVE_CONTACT and ACCOUNT_NAME in message and USER in message \
				and self.names[message[USER]] == client:
			self.database.remove_contact(message[USER], message[ACCOUNT_NAME])
			try:
				send_message(client, RESPONSE_200)
			except OSError:
				self.remove_client(client)
		
		# Если это запрос пользователей онлайн
		elif ACTION in message and message[ACTION] == ONLINE_USERS_REQUEST in message and \
				self.names[message[ACCOUNT_NAME]] == client:
			self.database.active_users_list()
			response = RESPONSE_202
			response[LIST_INFO] = [name[0] for name in self.database.active_users_list().count()]
			try:
				send_message(client, RESPONSE_202)
			except OSError:
				self.remove_client(client)
		
		# Если это запрос известных пользователей
		elif ACTION in message and message[ACTION] == USERS_REQUEST and ACCOUNT_NAME in message \
				and self.names[message[ACCOUNT_NAME]] == client:
			response = RESPONSE_202
			response[LIST_INFO] = [user[0] for user in self.database.users_list()]
			try:
				send_message(client, response)
			except OSError:
				self.remove_client(client)
		
		# Если это запрос публичного ключа пользователя
		elif ACTION in message and message[ACTION] == PUBLIC_KEY_REQUEST and ACCOUNT_NAME in message:
			response = RESPONSE_511
			response[DATA] = self.database.get_pubkey(message[ACCOUNT_NAME])
			# может быть, что ключа ещё нет (пользователь никогда не логинился,
			# тогда шлём 400)
			if response[DATA]:
				try:
					send_message(client, response)
				except OSError:
					self.remove_client(client)
			else:
				response = RESPONSE_400
				response[ERROR] = 'Нет публичного ключа для данного пользователя'
				try:
					send_message(client, response)
				except OSError:
					self.remove_client(client)
		
		# Иначе отдаём Bad request
		else:
			response = RESPONSE_400
			response[ERROR] = 'Запрос некорректен.'
			try:
				send_message(client, response)
			except OSError:
				self.remove_client(client)
	
	def autorize_user(self, message, sock):
		"""
		Function that implements user authorization
		:param message:
		:param sock:
		:return:
		"""
		# Если имя пользователя уже занято то возвращаем 400
		server_log.debug(f'Start auth process for {message[USER]}')
		if message[USER][ACCOUNT_NAME] in self.names.keys():
			response = RESPONSE_400
			response[ERROR] = 'Имя пользователя уже занято.'
			try:
				server_log.debug(f'Username busy, sending {response}')
				send_message(sock, response)
			except OSError:
				server_log.debug('OS Error')
				pass
			self.clients.remove(sock)
			sock.close()
		# Проверяем что пользователь зарегистрирован на сервере.
		elif not self.database.check_user(message[USER][ACCOUNT_NAME]):
			response = RESPONSE_400
			response[ERROR] = 'Пользователь не зарегистрирован.'
			try:
				server_log.debug(f'Unknown username, sending {response}')
				send_message(sock, response)
			except OSError:
				pass
			self.clients.remove(sock)
			sock.close()
		else:
			server_log.debug('Correct username, starting passwd check.')
			# Иначе отвечаем 511 и проводим процедуру авторизации
			# Словарь - заготовка
			message_auth = RESPONSE_511
			# Набор байтов в hex представлении
			random_str = binascii.hexlify(os.urandom(64))
			# В словарь байты нельзя, декодируем (json.dumps -> TypeError)
			message_auth[DATA] = random_str.decode('ascii')
			# Создаём хэш пароля и связки с рандомной строкой, сохраняем
			# серверную версию ключа
			hash = hmac.new(self.database.get_hash(message[USER][ACCOUNT_NAME]), random_str, 'MD5')
			digest = hash.digest()
			server_log.debug(f'Auth message = {message_auth}')
			try:
				# Обмен с клиентом
				send_message(sock, message_auth)
				ans = get_message(sock)
			except OSError as err:
				server_log.debug('Error in auth, data:', exc_info=err)
				sock.close()
				return
			client_digest = binascii.a2b_base64(ans[DATA])
			# Если ответ клиента корректный, то сохраняем его в список
			# пользователей.
			if RESPONSE in ans and ans[RESPONSE] == 511 and hmac.compare_digest(
					digest, client_digest):
				self.names[message[USER][ACCOUNT_NAME]] = sock
				client_ip, client_port = sock.getpeername()
				try:
					send_message(sock, RESPONSE_200)
				except OSError:
					self.remove_client(message[USER][ACCOUNT_NAME])
				# добавляем пользователя в список активных и если у него изменился открытый ключ
				# сохраняем новый
				self.database.user_login(
					message[USER][ACCOUNT_NAME],
					client_ip,
					client_port,
					message[USER][PUBLIC_KEY])
			else:
				response = RESPONSE_400
				response[ERROR] = 'Неверный пароль.'
				try:
					send_message(sock, response)
				except OSError:
					pass
				self.clients.remove(sock)
				sock.close()
	
	def service_update_lists(self):
		"""
		Function that implements sending for clients a service message 205
		:return:
		"""
		for client in self.names:
			try:
				send_message(self.names[client], RESPONSE_205)
			except OSError:
				self.remove_client(self.names[client])



from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QDialog, QLabel, QComboBox, QPushButton


class DelUserDialog(QDialog):
	"""
	Class - dialog for selecting a contact to delete
	"""
	def __init__(self, database, server):
		super().__init__()
		self.database = database
		self.server = server
		
		self.setFixedSize(350, 120)
		self.setWindowTitle('Удаление пользователя')
		self.setAttribute(Qt.WA_DeleteOnClose)
		self.setModal(True)
		
		self.selector_label = QLabel(
			'Выберите пользователя для удаления:', self)
		self.selector_label.setFixedSize(200, 20)
		self.selector_label.move(10, 0)
		
		self.selector = QComboBox(self)
		self.selector.setFixedSize(200, 20)
		self.selector.move(10, 30)
		
		self.btn_ok = QPushButton('Удалить', self)
		self.btn_ok.setFixedSize(100, 30)
		self.btn_ok.move(230, 20)
		self.btn_ok.clicked.connect(self.remove_user)
		
		self.btn_cancel = QPushButton('Отмена', self)
		self.btn_cancel.setFixedSize(100, 30)
		self.btn_cancel.move(230, 60)
		self.btn_cancel.clicked.connect(self.close)
		
		self.all_users_fill()
	
	def all_users_fill(self):
		"""
		User list filling method
		:return:
		"""
		self.selector.addItems([item[0]
								for item in self.database.users_list()])
	
	def remove_user(self):
		"""
		Method - user delete handler
		:return:
		"""
		self.database.remove_user(self.selector.currentText())
		if self.selector.currentText() in self.server.names:
			sock = self.server.names[self.selector.currentText()]
			del self.server.names[self.selector.currentText()]
			self.server.remove_client(sock)
		# Рассылаем клиентам сообщение о необходимости обновить справочники
		self.server.service_update_lists()
		self.close()



import binascii
import hashlib

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QDialog, QPushButton, QLineEdit, QApplication, QLabel, QMessageBox


class RegisterUser(QDialog):
	"""
	Class dialog for user registration on the server.
	"""
	
	def __init__(self, database, server):
		super().__init__()
		
		self.database = database
		self.server = server
		
		self.setWindowTitle('Регистрация')
		self.setFixedSize(175, 183)
		self.setModal(True)
		self.setAttribute(Qt.WA_DeleteOnClose)
		
		self.label_username = QLabel('Введите имя пользователя:', self)
		self.label_username.move(10, 10)
		self.label_username.setFixedSize(150, 15)
		
		self.client_name = QLineEdit(self)
		self.client_name.setFixedSize(154, 20)
		self.client_name.move(10, 30)
		
		self.label_passwd = QLabel('Введите пароль:', self)
		self.label_passwd.move(10, 55)
		self.label_passwd.setFixedSize(150, 15)
		
		self.client_passwd = QLineEdit(self)
		self.client_passwd.setFixedSize(154, 20)
		self.client_passwd.move(10, 75)
		self.client_passwd.setEchoMode(QLineEdit.Password)
		self.label_conf = QLabel('Введите подтверждение:', self)
		self.label_conf.move(10, 100)
		self.label_conf.setFixedSize(150, 15)
		
		self.client_conf = QLineEdit(self)
		self.client_conf.setFixedSize(154, 20)
		self.client_conf.move(10, 120)
		self.client_conf.setEchoMode(QLineEdit.Password)
		
		self.btn_ok = QPushButton('Сохранить', self)
		self.btn_ok.move(10, 150)
		self.btn_ok.clicked.connect(self.save_data)
		
		self.btn_cancel = QPushButton('Выход', self)
		self.btn_cancel.move(90, 150)
		self.btn_cancel.clicked.connect(self.close)
		
		self.messages = QMessageBox()
		
		self.show()
	
	def save_data(self):
		"""
		Method for checking the correctness of entering and saving a new user to the database.
		:return:
		"""
		if not self.client_name.text():
			self.messages.critical(
				self, 'Ошибка', 'Не указано имя пользователя.')
			return
		elif self.client_passwd.text() != self.client_conf.text():
			self.messages.critical(
				self, 'Ошибка', 'Введённые пароли не совпадают.')
			return
		elif self.database.check_user(self.client_name.text()):
			self.messages.critical(
				self, 'Ошибка', 'Пользователь уже существует.')
			return
		else:
			# Генерируем хэш пароля, в качестве соли будем использовать логин в
			# нижнем регистре.
			passwd_bytes = self.client_passwd.text().encode('utf-8')
			salt = self.client_name.text().lower().encode('utf-8')
			passwd_hash = hashlib.pbkdf2_hmac(
				'sha512', passwd_bytes, salt, 10000)
			self.database.add_user(
				self.client_name.text(),
				binascii.hexlify(passwd_hash))
			self.messages.information(
				self, 'Успех', 'Пользователь успешно зарегистрирован.')
			# Рассылаем клиентам сообщение о необходимости обновить справичники
			self.server.service_update_lists()
			self.close()


if __name__ == '__main__':
	app = QApplication([])
	app.setAttribute(Qt.AA_DisableWindowContextHelpButton)
	dial = RegisterUser(None, None)
	app.exec_()



# -*- coding: utf-8 -*-
import argparse
import configparser
import os
import sys

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QApplication

from chatserver.сhatserver.Log.Log_decorators import log
from chatserver.сhatserver.Log.server_log_config import server_log
from chatserver.сhatserver.Mainlib.variables import default_port
from chatserver.сhatserver.server.Server_DataBase import ServerDB
from chatserver.сhatserver.server.main_window import MainWindow
from chatserver.сhatserver.server.server_core import MessageProcessor


@log
def arg_parser(default_port, default_address):
	"""
	Parser for command line arguments
	:param default_port:
	:param default_address:
	:return:
	"""
	server_log.debug(
		f'Инициализация парсера аргументов коммандной строки: {sys.argv}')
	parser = argparse.ArgumentParser()
	parser.add_argument('-p', default=default_port, type=int, nargs='?')
	parser.add_argument('-a', default=default_address, nargs='?')
	namespace = parser.parse_args(sys.argv[1:])
	listen_address = namespace.a
	listen_port = namespace.p
	server_log.debug('Аргументы успешно загружены.')
	return listen_address, listen_port


@log
def config_load():
	"""
	Configuration ini file parser
	:return:
	"""
	config = configparser.ConfigParser()
	dir_path = os.getcwd()
	config.read(f"{dir_path}/{'server.ini'}")
	# Если конфиг файл загружен правильно, запускаемся, иначе конфиг по
	# умолчанию.
	if 'SETTINGS' in config:
		return config
	else:
		config.add_section('SETTINGS')
		config.set('SETTINGS', 'Default_port', str(default_port))
		config.set('SETTINGS', 'Listen_Address', '')
		config.set('SETTINGS', 'Database_path', '')
		config.set('SETTINGS', 'Database_file', 'server_dataBase.db3')
		return config


@log
def main():
	"""
	Main server function
	:return:
	"""
	# Загрузка файла конфигурации сервера
	config = config_load()
	
	# Загрузка параметров командной строки, если нет параметров, то задаём
	# значения по умоланию.
	listen_address, listen_port = arg_parser(
		config['SETTINGS']['default_port'], config['SETTINGS']['default_address'])
	
	# Инициализация базы данных
	database = ServerDB('server_dataBase.db3')
	
	# Создание экземпляра класса - сервера и его запуск:
	server = MessageProcessor(listen_address, listen_port, database)
	server.daemon = True
	server.start()
	
	# Создаём графическое окуружение для сервера:
	server_app = QApplication(sys.argv)
	server_app.setAttribute(Qt.AA_DisableWindowContextHelpButton)
	main_window = MainWindow(database, server, config)
	
	# Запускаем GUI
	server_app.exec_()
	
	# По закрытию окон останавливаем обработчик сообщений
	server.running = False


if __name__ == '__main__':
	main()



import logging
import socket

from chatserver.сhatserver.Log.server_log_config import server_log
from chatserver.сhatserver.Mainlib.variables import ACTION, PRESENCE


def log(func_to_log):
	"""Decorator function."""
	
	def log_saver(*args, **kwargs):
		ret = func_to_log(*args, **kwargs)
		server_log.debug(f'Была вызвана функция {func_to_log.__name__} c параметрами {args}, {kwargs}. '
						 f'Вызов из модуля {func_to_log.__module__}')
		return ret
	
	return log_saver


def login_required(func):
	"""
	A decorator that verifies that the client is authorized on the server.
	Checks that the passed socket object is in
	the list of authorized clients.
	Except for passing the dictionary-query
	for authorization. If the client is not authorized,
	throws a TypeError exception
	"""
	
	def checker(*args, **kwargs):
		# проверяем, что первый аргумент - экземпляр MessageProcessor
		# Импортить необходимо тут, иначе ошибка рекурсивного импорта.
		from chatserver.сhatserver.server.server_core import MessageProcessor
		if isinstance(args[0], MessageProcessor):
			found = False
			for arg in args:
				if isinstance(arg, socket.socket):
					# Проверяем, что данный сокет есть в списке names класса
					# MessageProcessor
					for client in args[0].names:
						if args[0].names[client] == arg:
							found = True
			
			# Теперь надо проверить, что передаваемые аргументы не presence
			# сообщение. Если presense, то разрешаем
			for arg in args:
				if isinstance(arg, dict):
					if ACTION in arg and arg[ACTION] == PRESENCE:
						found = True
			# Если не не авторизован и не сообщение начала авторизации, то
			# вызываем исключение.
			if not found:
				raise TypeError
		return func(*args, **kwargs)
	
	return checker



import datetime

from sqlalchemy import create_engine, Column, Integer, String, DateTime, ForeignKey, Text
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker
from sqlalchemy.sql import default_comparator


# Класс - база данных сервера

class ServerDB:
	"""
    The class is a wrapper for working with the server database.
    Uses SQLite database, implemented with
    SQLAlchemy ORM and use a declarative approach.
    """
	DB_Base = declarative_base()
	
	class AllUsers(DB_Base):
		"""
        Class to display all users
        """
		__tablename__ = 'Users'
		id = Column(Integer, primary_key=True)
		name = Column(String, unique=True)
		last_login = Column(DateTime)
		passwd_hash = Column(String)
		pubkey = Column(Text)
		
		def __init__(self, username, passwd_hash):
			self.name = username
			self.last_login = datetime.datetime.now()
			self.passwd_hash = passwd_hash
			self.pubkey = None
			self.id = None
	
	class ActiveUsers(DB_Base):
		"""
        Class displaying all active users
        """
		__tablename__ = 'Active_users'
		id = Column(Integer, primary_key=True)
		user = Column(String, ForeignKey('Users.id'), unique=True)
		ip_address = Column(String)
		port = Column(Integer)
		login_time = Column(DateTime)
		
		def __init__(self, user_id, ip_address, port, login_time):
			self.user = user_id
			self.ip_address = ip_address
			self.port = port
			self.login_time = login_time
			self.id = None
	
	class LoginHistory(DB_Base):
		"""
        Class to display login history.
        """
		__tablename__ = 'login_history'
		id = Column(Integer, primary_key=True)
		name = Column(String, ForeignKey('Users.id'), unique=True)
		date_time = Column(DateTime)
		ip = Column(String)
		port = Column(Integer)
		
		def __init__(self, name, date, ip, port):
			self.id = None
			self.name = name
			self.date_time = date
			self.ip = ip
			self.port = port
	
	# Класс - отображение таблицы контактов пользователей
	class UsersContacts(DB_Base):
		"""
        Class for displaying user contacts table
        """
		__tablename__ = 'Contacts'
		id = Column(Integer, primary_key=True)
		user = Column(String, ForeignKey('Users.id'))
		contact = Column(String, ForeignKey('Users.id'))
		
		def __init__(self, user, contact):
			self.id = None
			self.user = user
			self.contact = contact
	
	# Класс отображение таблицы истории действий (статистики) пользователя
	class UsersHistory:
		"""
        Class displaying the table of the history of actions (statistics) of the user
        """
		__tablename__ = 'History'
		id = Column(Integer, primary_key=True)
		user = Column(String, ForeignKey('Users.id'))
		sent = Column(Integer)
		accepted = Column(Integer)
		
		def __init__(self, user):
			self.id = None
			self.user = user
			self.sent = 0
			self.accepted = 0
	
	def __init__(self, path):
		self.engine = create_engine(f'sqlite:///{path}', echo=False, pool_recycle=7200,
									connect_args={'check_same_thread': False})
		
		self.DB_Base.metadata.create_all(self.engine)
		Session = sessionmaker(bind=self.engine)
		self.session = Session()
		print(Session)
		
		# Если в таблице активных пользователей есть записи, то их необходимо удалить
		# Когда устанавливаем соединение, очищаем таблицу активных пользователей
		self.session.query(self.ActiveUsers).delete()
		self.session.commit()
	
	def user_login(self, username, ip_address, port, key):
		"""
        The function executed when the user logs in, writes the fact of login to the database
        :param username:
        :param ip_address:
        :param port:
        :param key:
        :return:
        """
		# Запрос в таблицу пользователей на наличие там пользователя с таким именем
		rez = self.session.query(self.AllUsers).filter_by(name=username)
		print(rez)
		# Если имя пользователя уже присутствует в таблице, обновляем время последнего входа
		if rez.count():
			user = rez.first()
			user.last_login = datetime.datetime.now()
			if user.pubkey != key:
				user.pubkey = key
		# Если нет, то генерируем исключение
		else:
			raise ValueError('Пользователь не зарегистрирован.')
		
		# Теперь можно создать запись в таблицу активных пользователей о факте входа.
		# Создаем экземпляр класса self.ActiveUsers, через который передаем данные в таблицу
		new_active_user = self.ActiveUsers(user.id, ip_address, port, datetime.datetime.now())
		self.session.add(new_active_user)
		
		# и сохранить в историю входов
		# Создаем экземпляр класса self.LoginHistory, через который передаем данные в таблицу
		history = self.LoginHistory(user.id, datetime.datetime.now(), ip_address, port)
		self.session.add(history)
		
		# Сохраняем изменения
		self.session.commit()
	
	def user_logout(self, username):
		"""
        User disconnection locking function
        :param username:
        :return:
        """
		# Запрашиваем пользователя, что покидает нас
		# получаем запись из таблицы AllUsers
		user = self.session.query(self.AllUsers).filter_by(name=username).first()
		
		# Удаляем его из таблицы активных пользователей.
		# Удаляем запись из таблицы ActiveUsers
		self.session.query(self.ActiveUsers).filter_by(user=user.id).delete()
		
		# Применяем изменения
		self.session.commit()
	
	def users_list(self):
		"""
        The function returns a list of known users with the time of the last login
        :return:
        """
		query = self.session.query(
			self.AllUsers.name,
			self.AllUsers.last_login,
		)
		# Возвращаем список кортежей
		return query.all()
	
	def active_users_list(self):
		"""
        The function returns a list of active users
        :return:
        """
		# Запрашиваем соединение таблиц и собираем кортежи имя, адрес, порт, время.
		query = self.session.query(
			self.AllUsers.name,
			self.ActiveUsers.ip_address,
			self.ActiveUsers.port,
			self.ActiveUsers.login_time
		).join(self.AllUsers)
		# Возвращаем список кортежей
		return query.all()
	
	def login_history(self, username=None):
		"""
        Function that returns login history for one user or all users
        :param username:
        :return:
        """
		# Запрашиваем историю входа
		query = self.session.query(self.AllUsers.name,
								   self.LoginHistory.date_time,
								   self.LoginHistory.ip,
								   self.LoginHistory.port
								   ).join(self.AllUsers)
		# Если было указано имя пользователя, то фильтруем по нему
		if username:
			query = query.filter(self.AllUsers.name == username)
		return query.all()
	
	def process_message(self, sender, recipient):
		"""
        The function records the transmission of the message and makes the appropriate marks in the database
        :param sender:
        :param recipient:
        :return:
        """
		# Получаем ID отправителя и получателя
		sender = self.session.query(self.AllUsers).filter_by(name=sender).first().id
		recipient = self.session.query(self.AllUsers).filter_by(name=recipient).first().id
		# Запрашиваем строки из истории и увеличиваем счётчики
		sender_row = self.session.query(self.UsersHistory).filter_by(user=sender).first()
		sender_row.sent += 1
		recipient_row = self.session.query(self.UsersHistory).filter_by(user=recipient).first()
		recipient_row.accepted += 1
		
		self.session.commit()
	
	def add_user(self, name, passwd_hash):
		"""
        User registration function. Accepts a name and a password hash, creates an entry in the statistics table.
        :param name:
        :param passwd_hash:
        :return:
        """
		user_row = self.AllUsers(name, passwd_hash)
		self.session.add(user_row)
		self.session.commit()
		history_row = self.UsersHistory(user_row.id)
		self.session.add(history_row)
		self.session.commit()
	
	def remove_user(self, name):
		"""
        The function of removing a user from the database
        :param name:
        :return:
        """
		user = self.session.query(self.AllUsers).filter_by(name=name).first()
		self.session.query(self.ActiveUsers).filter_by(user=user.id).delete()
		self.session.query(self.LoginHistory).filter_by(name=user.id).delete()
		self.session.query(self.UsersContacts).filter_by(user=user.id).delete()
		self.session.query(
			self.UsersContacts).filter_by(
			contact=user.id).delete()
		self.session.query(self.UsersHistory).filter_by(user=user.id).delete()
		self.session.query(self.AllUsers).filter_by(name=name).delete()
		self.session.commit()
	
	def get_hash(self, name):
		"""
        Function for obtaining a hash of a user's password
        :param name:
        :return:
        """
		user = self.session.query(self.AllUsers).filter_by(name=name).first()
		return user.passwd_hash
	
	def get_pubkey(self, name):
		"""
        Function for obtaining the public key of a user
        :param name:
        :return:
        """
		user = self.session.query(self.AllUsers).filter_by(name=name).first()
		return user.pubkey
	
	def check_user(self, name):
		"""
        Function for checking user existence
        :param name:
        :return:
        """
		if self.session.query(self.AllUsers).filter_by(name=name).count():
			return True
		else:
			return False
	
	def user_is_online(self, user):
		""""""
		if self.session.query(self.ActiveUsers).filter_by(user=user).count():
			return True
		else:
			return False
	
	def add_contact(self, user, contact):
		"""
        The function adds a contact for the user
        :param user:
        :param contact:
        :return:
        """
		# Получаем ID пользователей
		user = self.session.query(self.AllUsers).filter_by(name=user).first()
		contact = self.session.query(self.AllUsers).filter_by(name=contact).first()
		
		# Проверяем что не дубль и что контакт может существовать (полю пользователь мы доверяем)
		if not contact or self.session.query(self.UsersContacts).filter_by(user=user.id, contact=contact.id).count():
			return
		
		# Создаём объект и заносим его в базу
		contact_row = self.UsersContacts(user.id, contact.id)
		self.session.add(contact_row)
		self.session.commit()
	
	def remove_contact(self, user, contact):
		"""
        The function removes a contact from the database
        :param user:
        :param contact:
        :return:
        """
		# Получаем ID пользователей
		user = self.session.query(self.AllUsers).filter_by(name=user).first()
		contact = self.session.query(self.AllUsers).filter_by(name=contact).first()
		
		# Проверяем что контакт может существовать (полю пользователь мы доверяем)
		if not contact:
			return
		
		# Удаляем требуемое
		print(self.session.query(self.UsersContacts).filter(
			self.UsersContacts.user == user.id,
			self.UsersContacts.contact == contact.id
		).delete())
		self.session.commit()
	
	def get_contacts(self, username):
		"""
        The function returns the user's contact list
        :param username:
        :return:
        """
		# Запрашивааем указанного пользователя
		user = self.session.query(self.AllUsers).filter_by(name=username).one()
		
		# Запрашиваем его список контактов
		query = self.session.query(self.UsersContacts, self.AllUsers.name).filter_by(user=user.id). \
			join(self.AllUsers, self.UsersContacts.contact == self.AllUsers.id)
		
		# выбираем только имена пользователей и возвращаем их.
		return [contact[1] for contact in query.all()]
	
	def message_history(self):
		"""
        The function returns the number of messages sent and received
        :return:
        """
		query = self.session.query(
			self.AllUsers.name,
			self.AllUsers.last_login,
			self.UsersHistory.sent,
			self.UsersHistory.accepted
		).join(self.AllUsers)
		# Возвращаем список кортежей
		return query.all()


if __name__ == '__main__':
	test_db = ServerDB('../server_dataBase.db3')
	test_db.user_login('1111', '192.168.1.113', 8080, 123456)
	test_db.user_login('McG', '192.168.1.115', 8082, 654321)
	test_db.user_login('McG2', '192.168.1.114', 8081, 132435)
	# print(test_db.users_list())
	# print(test_db.active_users_list())
	# test_db.user_logout('McG')
	# print(test_db.login_history('re'))
	test_db.add_contact('test2', 'test1')
	test_db.add_contact('test1', 'test3')
	test_db.add_contact('test1', 'test6')
	test_db.remove_contact('test1', 'test3')
	test_db.process_message('McG2', '1111')
	print(test_db.message_history())



# -*- coding: utf-8 -*-
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QStandardItemModel, QStandardItem
from PyQt5.QtWidgets import QDialog, QPushButton, QTableView


class StatWindow(QDialog):
	"""
	Class - window with user statistics
	"""
	def __init__(self, database):
		super().__init__()
		
		self.database = database
		self.initUI()
	
	def initUI(self):
		"""
		Window settings
		:return:
		"""
		# Настройки окна:
		self.setWindowTitle('Статистика клиентов')
		self.setFixedSize(600, 700)
		self.setAttribute(Qt.WA_DeleteOnClose)
		
		# Кнапка закрытия окна
		self.close_button = QPushButton('Закрыть', self)
		self.close_button.move(250, 650)
		self.close_button.clicked.connect(self.close)
		
		# Лист с собственно статистикой
		self.stat_table = QTableView(self)
		self.stat_table.move(10, 10)
		self.stat_table.setFixedSize(580, 620)
		
		self.create_stat_model()
	
	def create_stat_model(self):
		"""
		Method implementing filling the table with message statistics
		:return:
		"""
		# Список записей из базы
		stat_list = self.database.message_history()
		
		# Объект модели данных:
		list = QStandardItemModel()
		list.setHorizontalHeaderLabels(
			['Имя Клиента', 'Последний раз входил', 'Сообщений отправлено', 'Сообщений получено'])
		for row in stat_list:
			user, last_seen, sent, recvd = row
			user = QStandardItem(user)
			user.setEditable(False)
			last_seen = QStandardItem(str(last_seen.replace(microsecond=0)))
			last_seen.setEditable(False)
			sent = QStandardItem(str(sent))
			sent.setEditable(False)
			recvd = QStandardItem(str(recvd))
			recvd.setEditable(False)
			list.appendRow([user, last_seen, sent, recvd])
		self.stat_table.setModel(list)
		self.stat_table.resizeColumnsToContents()
		self.stat_table.resizeRowsToContents()



from PyQt5.QtCore import QTimer
from PyQt5.QtGui import QStandardItemModel, QStandardItem
from PyQt5.QtWidgets import QMainWindow, QAction, qApp, QLabel, QTableView

from chatserver.сhatserver.server.add_user_window import RegisterUser
from chatserver.сhatserver.server.remove_user_window import DelUserDialog
from chatserver.сhatserver.server.server_conf_window import ConfigWindow
from chatserver.сhatserver.server.stat_window import StatWindow


class MainWindow(QMainWindow):
	"""
	Class - main server window
	"""
	def __init__(self, database, server, config):
		# Конструктор предка
		super().__init__()
		
		# База данных сервера
		self.database = database
		
		self.server_thread = server
		self.config = config
		
		# Ярлык выхода
		self.exitAction = QAction('Выход', self)
		self.exitAction.setShortcut('Ctrl+Q')
		self.exitAction.triggered.connect(qApp.quit)
		
		# Кнопка обновить список клиентов
		self.refresh_button = QAction('Обновить список', self)
		
		# Кнопка настроек сервера
		self.config_btn = QAction('Настройки сервера', self)
		
		# Кнопка регистрации пользователя
		self.register_btn = QAction('Регистрация пользователя', self)
		
		# Кнопка удаления пользователя
		self.remove_btn = QAction('Удаление пользователя', self)
		
		# Кнопка вывести историю сообщений
		self.show_history_button = QAction('История клиентов', self)
		
		# Статусбар
		self.statusBar()
		self.statusBar().showMessage('Server Working')
		
		# Тулбар
		self.toolbar = self.addToolBar('MainBar')
		self.toolbar.addAction(self.exitAction)
		self.toolbar.addAction(self.refresh_button)
		self.toolbar.addAction(self.show_history_button)
		self.toolbar.addAction(self.config_btn)
		self.toolbar.addAction(self.register_btn)
		self.toolbar.addAction(self.remove_btn)
		
		# Настройки геометрии основного окна
		# Поскольку работать с динамическими размерами мы не умеем, и мало
		# времени на изучение, размер окна фиксирован.
		self.setFixedSize(800, 600)
		self.setWindowTitle('Messaging Server alpha release')
		
		# Надпись о том, что ниже список подключённых клиентов
		self.label = QLabel('Список подключённых клиентов:', self)
		self.label.setFixedSize(240, 15)
		self.label.move(10, 25)
		
		# Окно со списком подключённых клиентов.
		self.active_clients_table = QTableView(self)
		self.active_clients_table.move(10, 45)
		self.active_clients_table.setFixedSize(780, 400)
		
		# Таймер, обновляющий список клиентов 1 раз в секунду
		self.timer = QTimer()
		self.timer.timeout.connect(self.create_users_model)
		self.timer.start(1000)
		
		# Связываем кнопки с процедурами
		self.refresh_button.triggered.connect(self.create_users_model)
		self.show_history_button.triggered.connect(self.show_statistics)
		self.config_btn.triggered.connect(self.server_config)
		self.register_btn.triggered.connect(self.reg_user)
		self.remove_btn.triggered.connect(self.rem_user)
		
		# Последним параметром отображаем окно.
		self.show()
	
	def create_users_model(self):
		"""
		Method filling the table of active users
		:return:
		"""
		list_users = self.database.active_users_list()
		list = QStandardItemModel()
		list.setHorizontalHeaderLabels(
			['Имя Клиента', 'IP Адрес', 'Порт', 'Время подключения'])
		for row in list_users:
			user, ip, port, time = row
			user = QStandardItem(user)
			user.setEditable(False)
			ip = QStandardItem(ip)
			ip.setEditable(False)
			port = QStandardItem(str(port))
			port.setEditable(False)
			# Уберём милисекунды из строки времени, т.к. такая точность не
			# требуется.
			time = QStandardItem(str(time.replace(microsecond=0)))
			time.setEditable(False)
			list.appendRow([user, ip, port, time])
		self.active_clients_table.setModel(list)
		self.active_clients_table.resizeColumnsToContents()
		self.active_clients_table.resizeRowsToContents()
	
	def show_statistics(self):
		"""
		Method that creates a window with customer statistics
		:return:
		"""
		global stat_window
		stat_window = StatWindow(self.database)
		stat_window.show()
	
	def server_config(self):
		"""
		Method that creates a window with server settings
		:return:
		"""
		global config_window
		# Создаём окно и заносим в него текущие параметры
		config_window = ConfigWindow(self.config)
	
	def reg_user(self):
		"""
		Method that creates a user registration window
		:return:
		"""
		global reg_window
		reg_window = RegisterUser(self.database, self.server_thread)
		reg_window.show()
	
	def rem_user(self):
		"""
		Method that creates a user deletion window
		:return:
		"""
		global rem_window
		rem_window = DelUserDialog(self.database, self.server_thread)
		rem_window.show()



default_port = 7777
default_address = '127.0.0.1'
# for JIM message
ACTION = 'action'
TIME = 'time'
USER = 'user'
ACCOUNT_NAME = 'account_name'
SENDER = 'from'
DESTINATION = 'to'
DATA = 'bin'
PUBLIC_KEY = 'pubkey'
PRESENCE = 'presence'
RESPONSE = 'response'
ERROR = 'error'
MESSAGE = 'message'
MESSAGE_TEXT = 'mess_text'
EXIT = 'exit'
GET_CONTACTS = 'get_contacts'
LIST_INFO = 'data_list'
REMOVE_CONTACT = 'remove'
ADD_CONTACT = 'add'
USERS_REQUEST = 'get_users'
PUBLIC_KEY_REQUEST = 'pubkey_need'
ONLINE_USERS_REQUEST = 'online_users_request'

RESPONSE_200 = {RESPONSE: 200}

RESPONSE_202 = {RESPONSE: 202,
                LIST_INFO: None
                }

RESPONSE_400 = {
    RESPONSE: 400,
    ERROR: None
}

RESPONSE_205 = {
	RESPONSE: 205
}

RESPONSE_511 = {
	RESPONSE: 511,
	DATA: None
}